@@ -4,9 +4,6 @@ Experiment and Store Classes
 ==========================================
 '''
 
-
-from __future__ import absolute_import, division, print_function
-
 import os
 import copy
 import math
@@ -22,7 +19,7 @@ import pprint
 from multiprocessing import Pool
 from typing import Any, Dict
 
-from vivarium.library.units import Quantity
+from vivarium.library.units import units, Quantity
 from vivarium.library.dict_utils import (
     deep_merge,
     deep_merge_multi_update,
@@ -35,6 +32,11 @@ from vivarium.core.process import (
     ParallelProcess,
     serialize_dictionary,
 )
+from vivarium.library.topology import (
+    get_in, delete_in, assoc_path,
+    without, update_in, inverse_topology,
+    path_list_to_dict,
+)
 from vivarium.core.registry import (
     divider_registry,
     updater_registry,
@@ -69,51 +71,6 @@ def key_for_value(d, looking):
             break
     return found
 
-
-def get_in(d, path, default=None):
-    if path:
-        head = path[0]
-        if head in d:
-            return get_in(d[head], path[1:])
-        return default
-    return d
-
-
-def assoc_path(d, path, value):
-    if path:
-        head = path[0]
-        if len(path) == 1:
-            d[head] = value
-        else:
-            if head not in d:
-                d[head] = {}
-            assoc_path(d[head], path[1:], value)
-    elif isinstance(value, dict):
-        deep_merge(d, value)
-
-
-def update_in(d, path, f):
-    if path:
-        head = path[0]
-        d.setdefault(head, {})
-        updated = copy.deepcopy(d)
-        updated[head] = update_in(d[head], path[1:], f)
-        return updated
-    return f(d)
-
-
-def delete_in(d, path):
-    if len(path) > 0:
-        head = path[0]
-        if len(path) == 1:
-            # at the node to be deleted
-            if head in d:
-                del d[head]
-        elif head in d:
-            # down = d[head]
-            delete_in(d[head], path[1:])
-
-
 def depth(tree, path=()):
     '''
     Create a mapping of every path in the tree to the node living at
@@ -132,12 +89,6 @@ def depth(tree, path=()):
     return base
 
 
-def without(d, removing):
-    return {
-        key: value
-        for key, value in d.items()
-        if key != removing}
-
 def schema_for(port, keys, initial_state, default=0.0, updater='accumulate'):
     return {
         key: {
@@ -307,7 +258,11 @@ class Store(object):
             if self.inner:
                 raise Exception('trying to assign leaf values to a branch at: {}'.format(self.path_for()))
             self.leaf = True
-            # self.units = config.get('_units', self.units)
+
+            if '_units' in config:
+                self.units = config['_units']
+                self.serializer = serializer_registry.access('units')
+
             if '_serializer' in config:
                 self.serializer = config['_serializer']
                 if isinstance(self.serializer, str):
@@ -316,8 +271,12 @@ class Store(object):
             if '_default' in config:
                 self.default = self.check_default(config.get('_default'))
                 if isinstance(self.default, Quantity):
-                    self.units = self.default.units
-                if isinstance(self.default, np.ndarray):
+                    self.units = self.units or self.default.units
+                    self.serializer = self.serializer or serializer_registry.access('units')
+                elif isinstance(self.default, list) and (len(self.default) > 0) and isinstance(self.default[0], Quantity):
+                    self.units = self.units or self.default[0].units
+                    self.serializer = self.serializer or serializer_registry.access('units')
+                elif isinstance(self.default, np.ndarray):
                     self.serializer = self.serializer or serializer_registry.access('numpy')
 
             if '_value' in config:
@@ -510,7 +469,12 @@ class Store(object):
         else:
             if self.emit:
                 if self.serializer:
-                    return self.serializer.serialize(self.value)
+                    if isinstance(self.value, list) and self.units:
+                        return self.serializer.serialize([v.to(self.units) for v in self.value])
+                    elif self.units:
+                        return self.serializer.serialize(self.value.to(self.units))
+                    else:
+                        return self.serializer.serialize(self.value)
                 elif isinstance(self.value, Process):
                     return self.value.pull_data()
                 else:
@@ -605,22 +569,34 @@ class Store(object):
             if self.value is None:
                 self.value = self.default
 
-    def apply_update(self, update, process_topology=None, state=None):
+    def apply_update(self, update, process_topology=None, state=None, experiment_topology=None):
         '''
         Given an arbitrary update, map all the values in that update
         to their positions in the tree where they apply, and update
         these values using each node's `_updater`.
 
-        There are five special update keys:
+        Arguments:
+            update: The update being applied.
+            process_topology: The topology of the process from which this update was passed.
+            state: The state at the start of the time step.
+            experiment_topology: The topology of the full experiment, with all processes and
+                their port mappings.
+
+        There are five topology update methods, which use the following special update keys:
 
-        * `_updater` - Override the default updater with any updater you want.
-        * `_delete` - The value here is a list of paths (tuples) to delete from
-          the tree.
         * `_add` - Adds states into the subtree, given a list of dicts containing:
 
             * path - Path to the added state key.
             * state - The value of the added state.
 
+        * `_move` - Moves a node from a source to a target location in the tree.
+            This uses an update to an :term:`outer` port, which contains both the
+            source and target node locations. Can move multiple nodes according to
+            a list of dicts containing:
+
+            * source - the source path from an outer process port
+            * target - the location where the node will be placed.
+
         * `_generate` - The value has four keys, which are essentially
           the arguments to the `generate()` function:
 
@@ -636,6 +612,14 @@ class Store(object):
             * daughters - List of two new daughter generate directives, of the
                 same form as the `_generate` value above.
 
+        * `_delete` - The value here is a list of paths (tuples) to delete from
+          the tree.
+
+
+        Additional special update keys are used for different update operations:
+
+        * `_updater` - Override the default updater with any updater you want.
+
         * `_reduce` - This allows a reduction over the entire subtree from some
           point downward. Its three keys are:
 
@@ -658,7 +642,7 @@ class Store(object):
             multi_update = update[MULTI_UPDATE_KEY]
             assert isinstance(multi_update, list)
             for update_value in multi_update:
-                self.apply_update(update_value, process_topology, state)
+                self.apply_update(update_value, process_topology, state, experiment_topology)
             return EMPTY_UPDATES
 
         elif self.inner or self.subschema:
@@ -667,25 +651,50 @@ class Store(object):
             process_updates, topology_updates, deletions = {}, {}, []
             update = dict(update)  # avoid mutating the caller's dict
 
-            delete_paths = update.pop('_delete', None)
-            if delete_paths is not None:
-                # delete a list of paths
-                here = self.path_for()
-                for path in delete_paths:
-                    self.delete_path(path)
-                    deletions.append(tuple(here + path))
-
             add_entries = update.pop('_add', None)
             if add_entries is not None:
                 # add a list of sub-states
                 for added in add_entries:
                     path = added['path']
-                    state = added['state']
+                    added_state = added['state']
 
                     target = self.establish_path(path, {})
                     self.apply_subschema_path(path)
                     target.apply_defaults()
-                    target.set_value(state)
+                    target.set_value(added_state)
+
+            move_entries = update.pop('_move', None)
+            if move_entries is not None:
+                # move nodes from source to target path
+                for move in move_entries:
+                    source_path = move['source']
+                    target_path = move['target'] + (source_path[-1],)
+                    here = self.path_for()
+                    source_absolute = tuple(here + source_path)
+
+                    # get the source node
+                    source_node = self.get_path(source_path)
+
+                    # get the source processes and topology
+                    source_process_paths = source_node.depth(filter=lambda x: isinstance(x.value, Process))
+                    source_processes = path_list_to_dict(source_process_paths, lambda x: x.value)
+                    source_topology = get_in(experiment_topology, source_absolute)
+
+                    # move source node to target path
+                    target = self.add_node(target_path, source_node)
+
+                    assoc_path(
+                        process_updates,
+                        target_path,
+                        source_processes)
+
+                    assoc_path(
+                        topology_updates,
+                        target_path,
+                        source_topology)
+
+                    self.delete_path(source_path)
+                    deletions.append(source_absolute)
 
             generate_entries = update.pop('_generate', None)
             if generate_entries is not None:
@@ -720,15 +729,14 @@ class Store(object):
                 initial_state = self.inner[mother].get_value(
                     condition=lambda child: not (isinstance(child.value, Process)),
                     f=lambda child: copy.deepcopy(child))
-                states = self.inner[mother].divide_value()
+                daughter_states = self.inner[mother].divide_value()
 
-                for daughter, state in zip(daughters, states):
-                    # daughter_id = daughter['daughter']
+                for daughter, daughter_state in zip(daughters, daughter_states):
 
                     # use initial state as default, merge in divided values
                     initial_state = deep_merge(
                         initial_state,
-                        state)
+                        daughter_state)
 
                     path = daughter['path']
 
@@ -758,11 +766,19 @@ class Store(object):
                 self.delete_path(mother_path)
                 deletions.append(tuple(here + mother_path))
 
+            delete_paths = update.pop('_delete', None)
+            if delete_paths is not None:
+                # delete a list of paths
+                here = self.path_for()
+                for path in delete_paths:
+                    self.delete_path(path)
+                    deletions.append(tuple(here + path))
+
             for key, value in update.items():
                 if key in self.inner:
                     inner = self.inner[key]
                     inner_topology, inner_processes, inner_deletions = inner.apply_update(
-                        value, process_topology, state)
+                        value, process_topology, state, experiment_topology)
 
                     if inner_topology:
                         topology_updates = deep_merge(
@@ -899,24 +915,22 @@ class Store(object):
                 key: state.inner_value(key)
                 for key in keys}
 
-    def depth(self, path=()):
+    def depth(self, path=(), filter=None):
         '''
         Create a mapping of every path in the tree to the node living at
-        that path in the tree.
+        that path in the tree. An optional `filter` argument is a function
+        that can declares the instances that will be returned, for example:
+        * filter=lambda x: isinstance(x.value, Process)
         '''
+        base = []
+        if filter is None or filter(self):
+            base += [(path, self)]
 
-        base = [(path, self)]
         for key, child in self.inner.items():
             down = tuple(path + (key,))
-            base += child.depth(down)
+            base += child.depth(down, filter)
         return base
 
-    def processes(self, path=()):
-        return {
-            path: state
-            for path, state in self.depth()
-            if state.value and isinstance(state.value, Process)}
-
     def apply_subschema_path(self, path):
         if path:
             inner = self.inner[path[0]]
@@ -1005,6 +1019,16 @@ class Store(object):
             self.apply_config(config, source=source)
             return self
 
+    def add_node(self, path, node):
+        ''' Add a node instance at the provided path '''
+        target = self.establish_path(path[:-1], {})
+        if target.get_value() and path[-1] in target.get_value():
+            # this path already exists, update it
+            self.apply_update({path[-1]: node.get_value()})
+        else:
+            target.inner.update({path[-1]: node})
+        return target
+
     def outer_path(self, path, source=None):
         '''
         Address a topology with the `_path` keyword if present,
@@ -1039,11 +1063,13 @@ class Store(object):
                 set(schema.keys()) - set(topology.keys()))
             if mismatch_topology:
                 raise Exception(
-                    'topology at path {} and source {} has keys that are not in the schema: {}'.format(
-                        self.path_for(), source, mismatch_topology))
+                    'topology for the process {} \n at path {} uses undeclared ports: {}'.format(
+                        source, self.path_for(), str(mismatch_topology)))
             if mismatch_schema:
-                log.debug('{} schema has keys not in topology: {}'.format(
-                    source, mismatch_schema))
+                log.info(
+                    'process {} has ports that are not included in the topology: {}'.format(
+                        source, mismatch_schema))
+
             for port, subschema in schema.items():
                 path = topology.get(port, (port,))
 
@@ -1115,85 +1141,6 @@ class Store(object):
         target.apply_defaults()
 
 
-def inverse_topology(outer, update, topology):
-    '''
-    Transform an update from the form its process produced into
-    one aligned to the given topology.
-
-    The inverse of this function (using a topology to construct a view for
-    the perspective of a Process ports_schema()) lives in `Store`, called
-    `topology_state`. This one stands alone as it does not require a store
-    to calculate.
-    '''
-
-    inverse = {}
-    for key, path in topology.items():
-        if key == '*':
-
-            if isinstance(path, dict):
-                node = inverse
-                if '_path' in path:
-                    inner = normalize_path(outer + path['_path'])
-                    node = get_in(inverse, inner)
-                    if node is None:
-                        node = {}
-                        assoc_path(inverse, inner, node)
-                    path = without(path, '_path')
-
-                for child, child_update in update.items():
-                    node[child] = inverse_topology(
-                        tuple(),
-                        update[child],
-                        path)
-
-            else:
-                for child, child_update in update.items():
-                    inner = normalize_path(outer + path + (child,))
-                    if isinstance(child_update, dict):
-                        inverse = update_in(
-                            inverse,
-                            inner,
-                            lambda current: deep_merge(
-                                current, child_update))
-                    else:
-                        assoc_path(inverse, inner, child_update)
-
-        elif key in update:
-            value = update[key]
-            if isinstance(path, dict):
-                node = inverse
-                if '_path' in path:
-                    inner = normalize_path(outer + path['_path'])
-                    node = get_in(inverse, inner)
-                    if node is None:
-                        node = {}
-                        assoc_path(inverse, inner, node)
-                    path = without(path, '_path')
-
-                    for update_key in update[key].keys():
-                        if update_key not in path:
-                            path[update_key] = (update_key,)
-
-                deep_merge(
-                    node,
-                    inverse_topology(
-                        tuple(),
-                        value,
-                        path))
-
-            else:
-                inner = normalize_path(outer + path)
-                if isinstance(value, dict):
-                    inverse = update_in(
-                        inverse,
-                        inner,
-                        lambda current: deep_merge_multi_update(current, value))
-                else:
-                    assoc_path(inverse, inner, value)
-
-    return inverse
-
-
 def invert_topology(update, args):
     path, topology = args
     return inverse_topology(path[:-1], update, topology)
@@ -1209,16 +1156,6 @@ def generate_state(processes, topology, initial_state):
     return state
 
 
-def normalize_path(path):
-    progress = []
-    for step in path:
-        if step == '..' and len(progress) > 0:
-            progress = progress[:-1]
-        else:
-            progress.append(step)
-    return progress
-
-
 def timestamp(dt=None):
     if not dt:
         dt = datetime.datetime.now()
@@ -1385,14 +1322,19 @@ class Experiment(object):
             'name': self.experiment_name,
             'description': self.description,
             'topology': self.topology,
-            # TODO -- handle large parameter sets in self.processes to meet mongoDB limit
-            # 'processes': serialize_dictionary(self.processes),
-            # 'state': self.state.get_config()
+            'processes': serialize_dictionary(self.processes),
+            'state': self.state.get_config()
         }
         emit_config = {
             'table': 'configuration',
             'data': data}
-        self.emitter.emit(emit_config)
+        try:
+            self.emitter.emit(emit_config)
+        except:
+            # TODO -- handle large parameter sets in self.processes to meet mongoDB limit
+            del emit_config['data']['processes']
+            del emit_config['data']['state']
+            self.emitter.emit(emit_config)
 
     def invoke_process(self, process, path, interval, states):
         if process.parallel:
@@ -1427,7 +1369,7 @@ class Experiment(object):
 
     def apply_update(self, update, process_topology, state):
         topology_updates, process_updates, deletions = self.state.apply_update(
-            update, process_topology, state)
+            update, process_topology, state, self.topology)
 
         if topology_updates:
             self.topology = deep_merge(self.topology, topology_updates)
@@ -1590,15 +1532,15 @@ class Experiment(object):
 
     def print_summary(self, clock_finish):
         if clock_finish < 1:
-            print('Completed in {:.6f} real seconds'.format(clock_finish))
+            print('Completed in {:.6f} seconds'.format(clock_finish))
         else:
-            print('Completed in {:.2f} real seconds'.format(clock_finish))
+            print('Completed in {:.2f} seconds'.format(clock_finish))
 
 def print_progress_bar(
         iteration,
         total,
         decimals=1,
-        length=60,
+        length=50,
 ):
     """ Call in a loop to create terminal progress bar
 
@@ -1617,6 +1559,181 @@ def print_progress_bar(
         print()
 
 # Tests
+quark_colors = ['green', 'red', 'blue']
+quark_spins = ['up', 'down']
+electron_spins = ['-1/2', '1/2']
+electron_orbitals = [
+    str(orbit) + 's'
+    for orbit in range(1, 8)]
+
+
+class Proton(Process):
+    name = 'proton'
+    defaults = {
+        'time_step': 1.0,
+        'radius': 0.0}
+
+    def __init__(self, parameters=None):
+        super(Proton, self).__init__(parameters)
+
+    def ports_schema(self):
+        return {
+            'radius': {
+                '_updater': 'set',
+                '_default': self.parameters['radius']},
+            'quarks': {
+                '_divider': 'split_dict',
+                '*': {
+                    'color': {
+                        '_updater': 'set',
+                        '_default': quark_colors[0]},
+                    'spin': {
+                        '_updater': 'set',
+                        '_default': quark_spins[0]}}},
+            'electrons': {
+                '*': {
+                    'orbital': {
+                        '_updater': 'set',
+                        '_default': electron_orbitals[0]},
+                    'spin': {
+                        '_default': electron_spins[0]}}}}
+
+    def next_update(self, timestep, states):
+        update = {}
+
+        collapse = np.random.random()
+        if collapse < states['radius'] * timestep:
+            update['radius'] = collapse
+            update['quarks'] = {}
+
+            for name, quark in states['quarks'].items():
+                update['quarks'][name] = {
+                    'color': np.random.choice(quark_colors),
+                    'spin': np.random.choice(quark_spins)}
+
+            update['electrons'] = {}
+            orbitals = electron_orbitals.copy()
+            for name, electron in states['electrons'].items():
+                np.random.shuffle(orbitals)
+                update['electrons'][name] = {
+                    'orbital': orbitals.pop()}
+
+        return update
+
+
+class Electron(Process):
+    name = 'electron'
+    defaults = {
+        'time_step': 1.0,
+        'spin': electron_spins[0]}
+
+    def __init__(self, parameters=None):
+        super(Electron, self).__init__(parameters)
+
+    def ports_schema(self):
+        return {
+            'spin': {
+                '_updater': 'set',
+                '_default': self.parameters['spin']},
+            'proton': {
+                'radius': {
+                    '_default': 0.0}}}
+
+    def next_update(self, timestep, states):
+        update = {}
+
+        if np.random.random() < states['proton']['radius']:
+            update['spin'] = np.random.choice(electron_spins)
+
+        return update
+
+
+class Sine(Process):
+    name = 'sine'
+    defaults = {
+        'initial_phase': 0.0}
+
+    def __init__(self, parameters=None):
+        super(Sine, self).__init__(parameters)
+
+    def ports_schema(self):
+        return {
+            'frequency': {
+                '_default': 440.0},
+            'amplitude': {
+                '_default': 1.0},
+            'phase': {
+                '_default': self.parameters['initial_phase']},
+            'signal': {
+                '_default': 0.0,
+                '_updater': 'set'}}
+
+    def next_update(self, timestep, states):
+        phase_shift = timestep * states['frequency'] % 1.0
+        signal = states['amplitude'] * math.sin(
+            2 * math.pi * (states['phase'] + phase_shift))
+
+        return {
+            'phase': phase_shift,
+            'signal': signal}
+
+
+def make_proton(parallel=False):
+    processes = {
+        'proton': Proton({'_parallel': parallel}),
+        'electrons': {
+            'a': {
+                'electron': Electron({'_parallel': parallel})},
+            'b': {
+                'electron': Electron()}}}
+
+    spin_path = ('internal', 'spin')
+    radius_path = ('structure', 'radius')
+
+    topology = {
+        'proton': {
+            'radius': radius_path,
+            'quarks': ('internal', 'quarks'),
+            'electrons': {
+                '_path': ('electrons',),
+                '*': {
+                    'orbital': ('shell', 'orbital'),
+                    'spin': spin_path}}},
+        'electrons': {
+            'a': {
+                'electron': {
+                    'spin': spin_path,
+                    'proton': {
+                        '_path': ('..', '..'),
+                        'radius': radius_path}}},
+            'b': {
+                'electron': {
+                    'spin': spin_path,
+                    'proton': {
+                        '_path': ('..', '..'),
+                        'radius': radius_path}}}}}
+
+    initial_state = {
+        'structure': {
+            'radius': 0.7},
+        'internal': {
+            'quarks': {
+                'x': {
+                    'color': 'green',
+                    'spin': 'up'},
+                'y': {
+                    'color': 'red',
+                    'spin': 'up'},
+                'z': {
+                    'color': 'blue',
+                    'spin': 'down'}}}}
+
+    return {
+        'processes': processes,
+        'topology': topology,
+        'initial_state': initial_state}
+
+
 def test_recursive_store():
     environment_config = {
         'environment': {
@@ -1695,190 +1812,6 @@ def test_recursive_store():
     state.state_for(['environment'], ['temperature'])
 
 
-def test_in():
-    blank = {}
-    path = ['where', 'are', 'we']
-    assoc_path(blank, path, 5)
-    print(blank)
-    print(get_in(blank, path))
-    blank = update_in(blank, path, lambda x: x + 6)
-    print(blank)
-
-
-quark_colors = ['green', 'red', 'blue']
-quark_spins = ['up', 'down']
-electron_spins = ['-1/2', '1/2']
-electron_orbitals = [
-    str(orbit) + 's'
-    for orbit in range(1, 8)]
-
-
-class Proton(Process):
-    name = 'proton'
-    defaults = {
-        'time_step': 1.0,
-        'radius': 0.0}
-
-    def __init__(self, parameters=None):
-        super(Proton, self).__init__(parameters)
-
-    def ports_schema(self):
-        return {
-            'radius': {
-                '_updater': 'set',
-                '_default': self.parameters['radius']},
-            'quarks': {
-                '_divider': 'split_dict',
-                '*': {
-                    'color': {
-                        '_updater': 'set',
-                        '_default': quark_colors[0]},
-                    'spin': {
-                        '_updater': 'set',
-                        '_default': quark_spins[0]}}},
-            'electrons': {
-                '*': {
-                    'orbital': {
-                        '_updater': 'set',
-                        '_default': electron_orbitals[0]},
-                    'spin': {
-                        '_default': electron_spins[0]}}}}
-
-    def next_update(self, timestep, states):
-        update = {}
-
-        collapse = np.random.random()
-        if collapse < states['radius'] * timestep:
-            update['radius'] = collapse
-            update['quarks'] = {}
-
-            for name, quark in states['quarks'].items():
-                update['quarks'][name] = {
-                    'color': np.random.choice(quark_colors),
-                    'spin': np.random.choice(quark_spins)}
-
-            update['electrons'] = {}
-            orbitals = electron_orbitals.copy()
-            for name, electron in states['electrons'].items():
-                np.random.shuffle(orbitals)
-                update['electrons'][name] = {
-                    'orbital': orbitals.pop()}
-
-        return update
-
-class Electron(Process):
-    name = 'electron'
-    defaults = {
-        'time_step': 1.0,
-        'spin': electron_spins[0]}
-
-    def __init__(self, parameters=None):
-        super(Electron, self).__init__(parameters)
-
-    def ports_schema(self):
-        return {
-            'spin': {
-                '_updater': 'set',
-                '_default': self.parameters['spin']},
-            'proton': {
-                'radius': {
-                    '_default': 0.0}}}
-
-    def next_update(self, timestep, states):
-        update = {}
-
-        if np.random.random() < states['proton']['radius']:
-            update['spin'] = np.random.choice(electron_spins)
-
-        return update
-
-
-def make_proton(parallel=False):
-    processes = {
-        'proton': Proton({'_parallel': parallel}),
-        'electrons': {
-            'a': {
-                'electron': Electron({'_parallel': parallel})},
-            'b': {
-                'electron': Electron()}}}
-
-    spin_path = ('internal', 'spin')
-    radius_path = ('structure', 'radius')
-
-    topology = {
-        'proton': {
-            'radius': radius_path,
-            'quarks': ('internal', 'quarks'),
-            'electrons': {
-                '_path': ('electrons',),
-                '*': {
-                    'orbital': ('shell', 'orbital'),
-                    'spin': spin_path}}},
-        'electrons': {
-            'a': {
-                'electron': {
-                    'spin': spin_path,
-                    'proton': {
-                        '_path': ('..', '..'),
-                        'radius': radius_path}}},
-            'b': {
-                'electron': {
-                    'spin': spin_path,
-                    'proton': {
-                        '_path': ('..', '..'),
-                        'radius': radius_path}}}}}
-
-    initial_state = {
-        'structure': {
-            'radius': 0.7},
-        'internal': {
-            'quarks': {
-                'x': {
-                    'color': 'green',
-                    'spin': 'up'},
-                'y': {
-                    'color': 'red',
-                    'spin': 'up'},
-                'z': {
-                    'color': 'blue',
-                    'spin': 'down'}}}}
-
-    return {
-        'processes': processes,
-        'topology': topology,
-        'initial_state': initial_state}
-
-
-class Sine(Process):
-    name = 'sine'
-    defaults = {
-        'initial_phase': 0.0}
-
-    def __init__(self, parameters=None):
-        super(Sine, self).__init__(parameters)
-
-    def ports_schema(self):
-        return {
-            'frequency': {
-                '_default': 440.0},
-            'amplitude': {
-                '_default': 1.0},
-            'phase': {
-                '_default': self.parameters['initial_phase']},
-            'signal': {
-                '_default': 0.0,
-                '_updater': 'set'}}
-
-    def next_update(self, timestep, states):
-        phase_shift = timestep * states['frequency'] % 1.0
-        signal = states['amplitude'] * math.sin(
-            2 * math.pi * (states['phase'] + phase_shift))
-
-        return {
-            'phase': phase_shift,
-            'signal': signal}
-
-
 def test_topology_ports():
     proton = make_proton()
 
@@ -1964,37 +1897,58 @@ def test_timescales():
     experiment.update(10.0)
 
 
-def test_inverse_topology():
-    update = {
-        'port1': {
-            'a': 5},
-        'port2': {
-            'b': 10},
-        'port3': {
-            'b': 10},
-        'global': {
-            'c': 20}}
+def test_2_store_1_port():
+    """
+    Split one port of a processes into two stores
+    """
+    class OnePort(Process):
+        name = 'one_port'
+        def ports_schema(self):
+            return {
+                'A': {
+                    'a': {
+                        '_default': 0,
+                        '_emit': True},
+                    'b': {
+                        '_default': 0,
+                        '_emit': True}
+                }
+            }
+        def next_update(self, timestep, states):
+            return {
+                'A': {
+                    'a': 1,
+                    'b': 2}}
 
-    topology = {
-        'port1': ('boundary', 'x'),
-        'global': ('boundary',),
-        'port2': ('boundary', 'y'),
-        'port3': ('boundary', 'x')}
+    class SplitPort(Generator):
+        """splits OnePort's ports into two stores"""
+        name = 'split_port_generator'
+        def generate_processes(self, config):
+            return {
+                'one_port': OnePort({})}
+        def generate_topology(self, config):
+            return {
+                'one_port': {
+                    'A': {
+                        'a': ('internal', 'a',),
+                        'b': ('external', 'a',)
+                    }
+                }}
 
-    path = ('agent',)
-    inverse = inverse_topology(path, update, topology)
-    expected_inverse = {
-        'agent': {
-            'boundary': {
-                'x': {
-                    'a': 5,
-                    'b': 10},
-                'y': {
-                    'b': 10},
-                'c': 20}}}
-
-    assert inverse == expected_inverse
+    # run experiment
+    split_port = SplitPort({})
+    network = split_port.generate()
+    exp = Experiment({
+        'processes': network['processes'],
+        'topology': network['topology']})
 
+    exp.update(2)
+    output = exp.emitter.get_timeseries()
+    expected_output = {
+        'external': {'a': [0, 2, 4]},
+        'internal': {'a': [0, 1, 2]},
+        'time': [0.0, 1.0, 2.0]}
+    assert output == expected_output
 
 
 def test_multi_port_merge():
@@ -2182,124 +2136,6 @@ def test_parallel():
     experiment.end()
 
 
-class TestUpdateIn:
-    d = {
-        'foo': {
-            1: {
-                'a': 'b',
-            },
-        },
-        'bar': {
-            'c': 'd',
-        },
-    }
-
-    def test_simple(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated, ('foo', 1, 'a'), lambda current: 'updated')
-        expected = {
-            'foo': {
-                1: {
-                    'a': 'updated',
-                },
-            },
-            'bar': {
-                'c': 'd',
-            },
-        }
-        assert updated == expected
-
-    def test_add_leaf(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated, ('foo', 1, 'new'), lambda current: 'updated')
-        expected = {
-            'foo': {
-                1: {
-                    'a': 'b',
-                    'new': 'updated',
-                },
-            },
-            'bar': {
-                'c': 'd',
-            },
-        }
-        assert updated == expected
-
-    def test_add_dict(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated, ('foo', 2), lambda current: {'a': 'updated'})
-        expected = {
-            'foo': {
-                1: {
-                    'a': 'b',
-                },
-                2: {
-                    'a': 'updated',
-                },
-            },
-            'bar': {
-                'c': 'd',
-            },
-        }
-        assert updated == expected
-
-    def test_complex_merge(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated, ('foo',),
-            lambda current: deep_merge(
-                current,
-                {'foo': {'a': 'updated'}, 'b': 2}),
-            )
-        expected = {
-            'foo': {
-                'foo': {
-                    'a': 'updated',
-                },
-                'b': 2,
-                1: {
-                    'a': 'b',
-                },
-            },
-            'bar': {
-                'c': 'd',
-            },
-        }
-        assert updated == expected
-
-    def test_add_to_root(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated,
-            tuple(),
-            lambda current: deep_merge(current, ({'a': 'updated'})),
-        )
-        expected = {
-            'foo': {
-                1: {
-                    'a': 'b',
-                },
-            },
-            'bar': {
-                'c': 'd',
-            },
-            'a': 'updated'
-        }
-        assert updated == expected
-
-    def test_set_root(self):
-        updated = copy.deepcopy(self.d)
-        updated = update_in(
-            updated, tuple(), lambda current: {'a': 'updated'})
-        expected = {
-            'a': 'updated',
-        }
-        assert updated == expected
-
-
 def test_depth():
     nested = {
         'A': {
@@ -2314,19 +2150,6 @@ def test_depth():
     assert dissected[('A', 'AB', 'ABC')] == 11
 
 
-def test_deletion():
-    nested = {
-        'A': {
-            'AA': 5,
-            'AB': {
-                'ABC': 11}},
-        'B': {
-            'BA': 6}}
-
-    delete_in(nested, ('A', 'AA'))
-    assert 'AA' not in nested['A']
-
-
 def test_sine():
     sine = Sine()
     print(sine.next_update(0.25 / 440.0, {
@@ -2335,14 +2158,80 @@ def test_sine():
         'phase': 1.5}))
 
 
+def test_units():
+    class UnitsMicrometer(Process):
+        name = 'units_micrometer'
+        def ports_schema(self):
+            return {
+                'A': {
+                    'a': {
+                        '_default': 0 * units.um,
+                        '_emit': True},
+                    'b': {
+                        '_default': 'string b',
+                        '_emit': True,
+                    }
+                }
+            }
+        def next_update(self, timestep, states):
+            return {
+                'A': {'a': 1 * units.um}}
+    class UnitsMillimeter(Process):
+        name = 'units_millimeter'
+        def ports_schema(self):
+            return {
+                'A': {
+                    'a': {
+                        # '_default': 0 * units.mm,
+                        '_emit': True}}}
+        def next_update(self, timestep, states):
+            return {
+                'A': {'a': 1 * units.mm}}
+    class MultiUnits(Generator):
+        name = 'multi_units_generator'
+        def generate_processes(self, config):
+            return {
+                'units_micrometer': UnitsMicrometer({}),
+                'units_millimeter': UnitsMillimeter({})}
+        def generate_topology(self, config):
+            return {
+                'units_micrometer': {'A': ('aaa',)},
+                'units_millimeter': {'A': ('aaa',)}}
+
+    # run experiment
+    multi_unit = MultiUnits({})
+    network = multi_unit.generate()
+    exp = Experiment({
+        'processes': network['processes'],
+        'topology': network['topology']})
+
+    exp.update(5)
+    timeseries = exp.emitter.get_timeseries()
+    print('TIMESERIES')
+    pp(timeseries)
+
+    data = exp.emitter.get_data()
+    print('DATA')
+    pp(data)
+
+    data_deserialized = exp.emitter.get_data_deserialized()
+    print('DESERIALIZED')
+    pp(data_deserialized)
+
+    data_unitless = exp.emitter.get_data_unitless()
+    print('UNITLESS')
+    pp(data_unitless)
+
+
 if __name__ == '__main__':
     # test_recursive_store()
-    # test_in()
     # test_timescales()
     # test_topology_ports()
     # test_multi()
     # test_sine()
     # test_parallel()
     # test_complex_topology()
+    # test_multi_port_merge()
+    # test_2_store_1_port()
 
-    test_multi_port_merge()
+    test_units()
