@@ -20,7 +20,12 @@ def set_axes(ax, show_xaxis=False):
         ax.tick_params(bottom=False, labelbottom=False)
 
 
-def plot_simulation_output(timeseries_raw, settings={}, out_dir='out', filename='simulation'):
+def plot_simulation_output(
+        timeseries_raw,
+        settings={},
+        out_dir=None,
+        filename=None,
+):
     '''
     Plot simulation output, with rows organized into separate columns.
 
@@ -28,6 +33,8 @@ def plot_simulation_output(timeseries_raw, settings={}, out_dir='out', filename=
         timeseries (dict): This can be obtained from simulation output with convert_to_timeseries()
         settings (dict): Accepts the following keys:
 
+            * **column_width** (:py:class:`int`): the width (inches) of
+              each column in the figure
             * **max_rows** (:py:class:`int`): ports with more states
               than this number of states get wrapped into a new column
             * **remove_zeros** (:py:class:`bool`): if True, timeseries
@@ -42,13 +49,13 @@ def plot_simulation_output(timeseries_raw, settings={}, out_dir='out', filename=
               ``[('port_id', 'state_id')]`` for all states that will be
               highlighted, even if they are otherwise to be removed
     '''
-    os.makedirs(out_dir, exist_ok=True)
 
     plot_fontsize = 8
     plt.rc('font', size=plot_fontsize)
     plt.rc('axes', titlesize=plot_fontsize)
 
     # get settings
+    column_width = settings.get('column_width', 3)
     max_rows = settings.get('max_rows', 25)
     remove_zeros = settings.get('remove_zeros', True)
     remove_flat = settings.get('remove_flat', False)
@@ -100,7 +107,7 @@ def plot_simulation_output(timeseries_raw, settings={}, out_dir='out', filename=
     # make figure and plot
     n_cols = len(columns)
     n_rows = max(columns)
-    fig = plt.figure(figsize=(n_cols * 3, n_rows * 1))
+    fig = plt.figure(figsize=(n_cols * column_width, n_rows * column_width/3))
     grid = plt.GridSpec(n_rows, n_cols)
     row_idx = 0
     col_idx = 0
@@ -143,7 +150,13 @@ def plot_simulation_output(timeseries_raw, settings={}, out_dir='out', filename=
                 row_idx += 1
             ax.set_xlim([time_vec[0], time_vec[-1]])
 
-    # save figure
-    fig_path = os.path.join(out_dir, filename)
-    plt.subplots_adjust(wspace=0.8, hspace=1.0)
-    plt.savefig(fig_path, bbox_inches='tight')
+    if out_dir:
+        os.makedirs(out_dir, exist_ok=True)
+        if filename is None:
+            filename = 'simulation'
+        # save figure
+        fig_path = os.path.join(out_dir, filename)
+        plt.subplots_adjust(wspace=column_width/3, hspace=column_width/3)
+        plt.savefig(fig_path, bbox_inches='tight')
+    else:
+        return fig
